; ---------------------------------------------------------------------
;
; The Decider - an executive decision maker
;
;	Copyright  Howard Sandford February 1997, all rights reserved
;                 hsandford@lowndes.co.uk
;			hfs@cix.co.uk
;
; ---------------------------------------------------------------------

; Appl sets the applications name and four character ID.  As yet this ID has not been
; registered, so it might overwrite an existing application on your Pilot

	Appl    "Decider", 'HSDM'

; Pilot.inc contains PalmOS constants, structure offsets, and API trap codes.

	include "Pilot.inc"


; Startup.inc contains a standard startup function. This function must be
; the first within an application and is called by the PalmOS after the app
; is loaded. The startup function in Startup.inc (__Startup__) calls the
; application-defined function PilotMain.

	include "Startup.inc"

; execyn.inc contains application specific constants and resource id's
	include "execyn.inc"


data
; gPosition keeps track of the position of the arrow bitmap.  1=North, 2=North West, 3=West, 
; 4=South West, 5=South, 6=South East, 7=East, 8=North East
	global	gPosition.w



code


; ---------------------------------------------------------------------
;
; PilotMain gets called by the startup code and is the equivalent of
; the main() function in C
;
;	cmd		- specifies how to launch the application
;	cmdPBP 	- parameter block for the command
;	launchFlags - flags used to configure the launch
; 
; ---------------------------------------------------------------------

proc PilotMain(cmd.w, cmdPBP.l, launchFlags.w)
local   err.w
local   evt.EventType

beginproc
; has the application started from a normal launch?
	cmp.w		#sysAppLaunchCmdNormalLaunch,cmd(a6)
	bne.s		PilotMainReturn

; Initialise the application by calling StartApplication
	call 		StartApplication()

; I haven't implemented any testing to see if the application has started successfully
; at the moment, everything is just assumed, but we could test just here ...
;	tst.b		d0							; Successful?
;	bne.s  	PilotMainReturn					; No, don't continue with application

	systrap FrmGotoForm(#mainForm.w)
		
	Call		EventLoop()

; tidy up code can be placed in StopApplication - none is used in this application
;	Call		StopApplication()

PilotMainReturn
	clr.l		d0
endproc




; ---------------------------------------------------------------------
;
; StartApplication initialises any variables etc that need to be set at
; startup
;
;	returns 0 if successful
;
; ---------------------------------------------------------------------

proc StartApplication()
beginproc
; the arrow points to the North on startup	
	move.w	#1, gPosition(a5)

; success, return 0
	clr.w		d0	
endproc




; ---------------------------------------------------------------------
;
; EventLoop takes events from the event manager and passes them on to
; the various handlers.
;
; ---------------------------------------------------------------------

proc	EventLoop()
local	event.EventType
local	error.w
beginproc	
EventLoopStart

; Get the next available event.
	systrap	EvtGetEvent(&event(a6).l, #evtWaitForever.l)
		
; Give the system a chance to handle the event.
	systrap	SysHandleEvent(&event(a6).l)
	tst.b		d0
	bne.s		EventLoopHandled

; Give the menu bar a chance to update and handle the event.	
	systrap	MenuHandleEvent(#0.l, &event(a6).l, &error(a6).l)
	tst.b		d0
	bne.s		EventLoopHandled

; Give the application a chance to handle the event.
	call		ApplicationHandleEvent(&event(a6).l)
	tst.b		d0
	bne.s		EventLoopHandled

; Let the form object provide default handling of the event.
	systrap	FrmDispatchEvent(&event(a6).l)
		
EventLoopHandled
; have we finished yet?
	cmp.w		#appStopEvent, event+EventType.eType(a6)
	bne		EventLoopStart
endproc




; ---------------------------------------------------------------------
;
; ApplicationHandleEvent loads a form resource and sets the event handler
; for the form
;
;	pevt - pointer to an EventType structure
;
;	returns 1 (TRUE) if the event has been handled
;	returns 0 (FALSE) if the event has not been handled
;
; ---------------------------------------------------------------------

proc ApplicationHandleEvent(pevt.l)
local frm.l
local formId.w
beginproc
	movea.l	pevt(a6),a0             ;a0 = pevt

; Handle frmLoadEvents
	cmpi.w	#frmLoadEvent,EventType.eType(a0) 	;frmLoadEvent?
      bne.s		ApplicationHandleEventNotHandled  	;no

; Load the form resource specified in the event,
; then activate the form.
	move.w	EventType.data+frmLoad.formID(a0), formId(a6)
		
; frm = FrmInitForm(formId);
	systrap	FrmInitForm(formId(a6).w)
	move.l 	a0, frm(a6)
	systrap	FrmSetActiveForm(frm(a6).l);

; Set the event handler for the form.  The handler of the currently 
; active form is called by FrmDispatchEvent each time it receives an event.
	cmpi.w 	#mainForm, formId(a6)
	bne.s		ApplicationHandleEventNotHandled

; Main form
	systrap	FrmSetEventHandler(frm(a6).l, &MainViewHandleEvent(pc).l);
;	bra.s		ApplicationHandleEventHandled


ApplicationHandleEventHandled
        moveq.l #1,d0                   ;event handled
        bra.s   ApplicationHandleEventReturn

ApplicationHandleEventNotHandled
        clr.b   d0

ApplicationHandleEventReturn
endproc




; ---------------------------------------------------------------------
;
; MainViewHandleEvent processes the events for the main form
;
;	pevt - pointer to an EventType structure
;
;	returns 1 (TRUE) if the event is handled
;	returns 0 (FALSE) if the event is not handled
;
; ---------------------------------------------------------------------

proc MainViewHandleEvent(pevt.l)
beginproc
	movea.l 	pevt(a6),a0             ;a0 = pevt
	move.w	EventType.eType(a0),d0	;d0 = eType

; Handle frmOpenEvent
	cmp.w		#frmOpenEvent,d0
	bne.s		MenuEventHandle

; Initialize the main form.
	call		MainViewInit()
	bra		MainFormHandleReturn

; Handle frmMenuEvent
MenuEventHandle
	cmp.w 	#menuEvent,d0
	bne.s		CtlEventHandle

; has "About" been chosen?
	cmp.w		#getInfoCmd,EventType.data+menu.itemID(a0)
	bne		MainFormHandleNotHandled

; Clear menu command prompt from the display
	systrap	MenuEraseStatus(#0)
				
; Load the info form, then display it
	movem.l 	a3,-(a7)                	;save registers we're going to trash
	systrap	FrmInitForm(#infoForm)      	; Returns pointer to form in a0
	move.l	a0,a3                   	; a3=frm
	systrap	FrmDoDialog(a3.l)

; Delete the info form
	systrap	FrmDeleteForm(a3.l)
	movem.l	(a7)+,a3
	bra		MainFormHandleHandled


; Handle ctlSelectEvent
; I used to have a spin button on the main form.  The code is here for future
; development
CtlEventHandle
	cmp.w		#ctlSelectEvent,d0
	bne.s		KeyDownEventHandle

; Is the Spin button being clicked?
	cmp.w		#mainSpinButton,EventType.data+ctlEnter.controlID(a0)
 	call		Spin()
	bra		MainFormHandleHandled

; Handle keydown events such as the "page up" key
KeyDownEventHandle
	cmp.w		#keyDownEvent,d0
	bne.s		MainFormHandleNotHandled

; Is the "page up" key being pressed?
	cmp.w		#pageUpChr, EventType.data+keyDown.chr(a0)
	bne		MainFormHandleNotHandled
	call		Spin()
;	bra		MainFormHandleHandled

MainFormHandleHandled
	moveq.l	#1, d0
	bra.s		MainFormHandleReturn
		
MainFormHandleNotHandled
	clr.b		d0

MainFormHandleReturn
endproc




; ---------------------------------------------------------------------
;
; MainViewInit initialises the main form
;
; ---------------------------------------------------------------------

proc	MainViewInit()
beginproc
; save the registers
	movem.l	a3,-(a7)

; Get a pointer to the main form.
	systrap FrmGetActiveForm()
	move 		a0, a3										; a3=frm

; Draw the form.
	systrap	FrmDrawForm(a3.l)

MainViewInitExit
; restore the registers
	movem.l	(a7)+,a3
endproc




; ---------------------------------------------------------------------
;
; Spin loops round and moves the arrow on one position every loop.  The
; number of loops is decided by the seconds value of the current time
;
; ---------------------------------------------------------------------

proc Spin()
beginproc
; save the registers we're going to use
	movem.l	d3-d4,-(a7)

; get the number of seconds right now, mask it with 31 and add another 31 for good measure
; this should give us between 31 and 62 spins depending on the current time
; TimGetSeconds actually gives us the number of seconds passed since 1/1/1904
	systrap	TimGetSeconds()
	andi		#31,d0
	add.w		#31,d0
	move.w	d0,d3

SpinLoop
	call		DrawArrow(gPosition(a5).w)

; draw the "inverse" bitmap for the text boxes - there must be a better way than
; using bitmaps ... XOR-ing a rectangle of screen addressed memory?  Can anyone
; point me at how to do this?
	call		DrawText(gPosition(a5).w)

	call		MakeSound()
      call		Delay(#mediumDelay.w)
	
; move the arrow position on one for the next drawing
	addq.w	#1, gPosition(a5)
	cmp.w		#9, gPosition(a5)
	bne.s		SpinContinue
	move.w	#1, gPosition(a5)

SpinContinue
	subq.w	#1,d3
	cmp.w		#0,d3
	bne		SpinLoop

; what if the arrow finishes on a corner position - add one to the counter and
; re-draw
	move.w	gPosition(a5),d4
	andi		#1, d4
	cmp.w		#1, d4
	beq		OneMoreGo
	bra		SpinReturn

OneMoreGo
	move.w	#1,d3
	bra		SpinLoop


SpinReturn
; the spinning has finished, so play a little sound
	call		PlayDecidedSound()

; restore the registers we've used
	movem.l	(a7)+,d3-d4

endproc




; ---------------------------------------------------------------------
;
; DrawArrow draws a suitable bitmap on the screen
;
; 	position - the direction (1 to 8) in which we want the bitmap to be drawn
;		     this parameter shouldn't really be needed as we've already got
;                the arrows current position from the global gPosition
;
; ---------------------------------------------------------------------

proc DrawArrow(position.w)
local		pBitmapPtr.l
local		pBitmapHnd.l
beginproc
; do I really need to set the draw window?
   	systrap	FrmGetActiveForm()
	systrap	FrmGetWindowHandle(a0.l)
	systrap	WinSetDrawWindow (a0.l)

; first direction - North?
Arrow1
	cmp.w		#1, position(a6)
	bne.s		Arrow2
	systrap	DmGetResource(#kidrTBMP.l,#bmArrow1.w)
	move.l	a0,pBitmapHnd(a6)
	bra		DrawTheBitmap		

; second direction - North-West?
Arrow2
	cmp.w		#2, position(a6)
	bne.s		Arrow3
	systrap	DmGetResource(#kidrTBMP.l,#bmArrow2.w)
	move.l	a0,pBitmapHnd(a6)
	bra		DrawTheBitmap		

; third direction - West?
Arrow3
	cmp.w		#3, position(a6)
	bne.s		Arrow4
	systrap	DmGetResource(#kidrTBMP.l,#bmArrow3.w)
	move.l	a0,pBitmapHnd(a6)
	bra		DrawTheBitmap		

; fourth direction - South-West?
Arrow4
	cmp.w		#4, position(a6)
	bne.s		Arrow5
	systrap	DmGetResource(#kidrTBMP.l,#bmArrow4.w)
	move.l	a0,pBitmapHnd(a6)
	bra		DrawTheBitmap		

; fifth direction - South?
Arrow5
	cmp.w		#5, position(a6)
	bne.s		Arrow6
	systrap	DmGetResource(#kidrTBMP.l,#bmArrow5.w)
	move.l	a0,pBitmapHnd(a6)
	bra		DrawTheBitmap		

; sixth direction - South-East?
Arrow6
	cmp.w		#6, position(a6)
	bne.s		Arrow7
	systrap	DmGetResource(#kidrTBMP.l,#bmArrow6.w)
	move.l	a0,pBitmapHnd(a6)
	bra		DrawTheBitmap		

; seventh direction - East?
Arrow7
	cmp.w		#7, position(a6)
	bne.s		Arrow8
	systrap	DmGetResource(#kidrTBMP.l,#bmArrow7.w)
	move.l	a0,pBitmapHnd(a6)
	bra		DrawTheBitmap		

; finally North-East?
Arrow8
	systrap	DmGetResource(#kidrTBMP.l,#bmArrow8.w)
	move.l	a0,pBitmapHnd(a6)


DrawTheBitmap
	systrap	MemHandleLock(pBitmapHnd(a6).l)
	move.l	a0,pBitmapPtr(a6)

	systrap	WinDrawBitmap(pBitmapPtr(a6).l,#30.w,#40.w)

; and clear up the resources/memory
	systrap	MemHandleUnlock(pBitmapHnd(a6).l)
	systrap	DmReleaseResource(pBitmapHnd(a6).l)
endproc




; ---------------------------------------------------------------------
;
; MakeSound clicks a 440Hz "A".  Some versions of Pilot.inc have to be 
; changed to make this work.  The structure SndCommandType should read as ...
;
;	struct SndCommandType
;	    cmd.b			<----- check that this reads as Byte
;	    padding.b		<----- check that the padding is present
;	    param1.l
;	    param2.w
;	    param3.w
;	endstruct
;
; ---------------------------------------------------------------------

proc MakeSound()
local sndStruct.SndCommandType
beginproc
	move.b	#sndCmdFreqDurationAmp, sndStruct.cmd(a6)
	move.l	#440, sndStruct.param1(a6)
	move.w	#1, sndStruct.param2(a6)
	move.w	#sndMaxAmp, sndStruct.param3(a6)
	systrap	SndDoCmd(#0.l,&sndStruct(a6),#0.w)
endproc




; ---------------------------------------------------------------------
;
; PlayDecidedSound plays a quick noise when the spinning has finished.  It
; was meant to be "E-B-D-B" but I couldn't work out what the frequencies ought
; to be.  This seems to sound ok for what I wanted.
;
; ---------------------------------------------------------------------

proc PlayDecidedSound()
local sndStruct.SndCommandType
beginproc
; play an E
	move.b	#sndCmdFreqDurationAmp, sndStruct.cmd(a6)
	move.l	#660, sndStruct.param1(a6)
	move.w	#100, sndStruct.param2(a6)
	move.w	#sndMaxAmp, sndStruct.param3(a6)
	systrap	SndDoCmd(#0.l,&sndStruct(a6),#0.w)

; play a B
	move.b	#sndCmdFreqDurationAmp, sndStruct.cmd(a6)
	move.l	#550, sndStruct.param1(a6)
	move.w	#100, sndStruct.param2(a6)
	move.w	#sndMaxAmp, sndStruct.param3(a6)
	systrap	SndDoCmd(#0.l,&sndStruct(a6),#0.w)

; play a D
	move.b	#sndCmdFreqDurationAmp, sndStruct.cmd(a6)
	move.l	#500, sndStruct.param1(a6)
	move.w	#100, sndStruct.param2(a6)
	move.w	#sndMaxAmp, sndStruct.param3(a6)
	systrap	SndDoCmd(#0.l,&sndStruct(a6),#0.w)

; play a B
	move.b	#sndCmdFreqDurationAmp, sndStruct.cmd(a6)
	move.l	#550, sndStruct.param1(a6)
	move.w	#100, sndStruct.param2(a6)
	move.w	#sndMaxAmp, sndStruct.param3(a6)
	systrap	SndDoCmd(#0.l,&sndStruct(a6),#0.w)
endproc




; ---------------------------------------------------------------------
;
; DrawText draws the filled in bitmaps for the text boxes.  There should be
; a way of automatically inverting the pixels such as XORing the screen mapped
; memory.  Can anyone point me at how to do this?
;
; 	position - the direction (1 to 8) in which we want the bitmap to be drawn
;		     this parameter shouldn't really be needed as we've already got
;                the arrows current position from the global gPosition
;
; ---------------------------------------------------------------------

proc DrawText(position.w)
local		pBitmapPtr.l
local		pBitmapHnd.l
beginproc
; do I really need to set the draw window?
	systrap	FrmGetActiveForm()
	systrap	FrmGetWindowHandle(a0.l)
	systrap	WinSetDrawWindow (a0.l)

; Yes box?
Text1
	cmp.w		#1, position(a6)
	bne.s		Text2
	systrap	DmGetResource(#kidrTBMP.l,#bmYesOn.w)
	move.l	a0,pBitmapHnd(a6)
	systrap	MemHandleLock(pBitmapHnd(a6).l)
	move.l	a0,pBitmapPtr(a6)
	systrap	WinDrawBitmap(pBitmapPtr(a6).l,#55.w,#20.w)
	bra		ClearUp

; reset the Yes box?
Text2
	cmp.w		#2, position(a6)
	bne.s		Text3
	systrap	DmGetResource(#kidrTBMP.l,#bmYesOff.w)
	move.l	a0,pBitmapHnd(a6)
	systrap	MemHandleLock(pBitmapHnd(a6).l)
	move.l	a0,pBitmapPtr(a6)
	systrap	WinDrawBitmap(pBitmapPtr(a6).l,#55.w,#20.w)
	bra		ClearUp

; Put if off to Tomorrow box?
Text3
	cmp.w		#3, position(a6)
	bne.s		Text4
	systrap	DmGetResource(#kidrTBMP.l,#bmTomorrowOn.w)
	move.l	a0,pBitmapHnd(a6)
	systrap	MemHandleLock(pBitmapHnd(a6).l)
	move.l	a0,pBitmapPtr(a6)
	systrap	WinDrawBitmap(pBitmapPtr(a6).l,#10.w,#40.w)
	bra		ClearUp

; Reset the Put if off to Tomorrow box?
Text4
	cmp.w		#4, position(a6)
	bne.s		Text5
	systrap	DmGetResource(#kidrTBMP.l,#bmTomorrowOff.w)
	move.l	a0,pBitmapHnd(a6)
	systrap	MemHandleLock(pBitmapHnd(a6).l)
	move.l	a0,pBitmapPtr(a6)
	systrap	WinDrawBitmap(pBitmapPtr(a6).l,#10.w,#40.w)
	bra		ClearUp

; No box?
Text5
	cmp.w		#5, position(a6)
	bne.s		Text6
	systrap	DmGetResource(#kidrTBMP.l,#bmNoOn.w)
	move.l	a0,pBitmapHnd(a6)
	systrap	MemHandleLock(pBitmapHnd(a6).l)
	move.l	a0,pBitmapPtr(a6)
	systrap	WinDrawBitmap(pBitmapPtr(a6).l,#55.w,#140.w)
	bra		ClearUp

; reset the No box?
Text6
	cmp.w		#6, position(a6)
	bne.s		Text7
	systrap	DmGetResource(#kidrTBMP.l,#bmNoOff.w)
	move.l	a0,pBitmapHnd(a6)
	systrap	MemHandleLock(pBitmapHnd(a6).l)
	move.l	a0,pBitmapPtr(a6)
	systrap	WinDrawBitmap(pBitmapPtr(a6).l,#55.w,#140.w)
	bra		ClearUp

; Pass the Buck box?
Text7
	cmp.w		#7, position(a6)
	bne.s		Text8
	systrap	DmGetResource(#kidrTBMP.l,#bmBuckOn.w)
	move.l	a0,pBitmapHnd(a6)
	systrap	MemHandleLock(pBitmapHnd(a6).l)
	move.l	a0,pBitmapPtr(a6)
	systrap	WinDrawBitmap(pBitmapPtr(a6).l,#130.w,#40.w)
	bra		ClearUp

; reset the Pass the Buck box?
Text8
	systrap	DmGetResource(#kidrTBMP.l,#bmBuckOff.w)
	move.l	a0,pBitmapHnd(a6)
	systrap	MemHandleLock(pBitmapHnd(a6).l)
	move.l	a0,pBitmapPtr(a6)
	systrap	WinDrawBitmap(pBitmapPtr(a6).l,#130.w,#40.w)

ClearUp
	systrap	MemHandleUnlock(pBitmapHnd(a6).l)
	systrap	DmReleaseResource(pBitmapHnd(a6).l)
endproc




; ---------------------------------------------------------------------
;
; Delay implements a simple delay by looping through a counter
;
;	delayValue - adjusts the number of times the loop is traversed
;
; ---------------------------------------------------------------------

proc Delay(delayValue.w)
beginproc
; save the registers
	movem.l	d3-d4,-(a7)

	move.w	delayValue(a6),d4

NumberOfTimesToLoop
	move.w	#$0fff,d3

DelayLoop
	subq.w	#1,d3
	cmp.w		#0,d3
	bne		DelayLoop

	subq.w	#1,d4
	cmp.w		#0,d4
	bne		NumberOfTimesToLoop		

; restore the registers
	movem.l	(a7)+,d3-d4
endproc



; ---------------------------------------------------------------------
;
; Resources
;
; ---------------------------------------------------------------------

; main icon for the application
	res 'WBMP', kidbAIB, "execyn.bmp"

; text box bitmaps
	res 'WBMP', bmYesOff, "bmyesoff.bmp"
	res 'WBMP', bmNoOff, "bmnooff.bmp"
	res 'WBMP', bmBuckOff, "bmbuckoff.bmp"
	res 'WBMP', bmTomorrowOff, "bmtomorrowoff.bmp"
	res 'WBMP', bmYesOn, "bmyeson.bmp"
	res 'WBMP', bmNoOn, "bmnoon.bmp"
	res 'WBMP', bmBuckOn, "bmbuckon.bmp"
	res 'WBMP', bmTomorrowOn, "bmtomorrowon.bmp"

; bitmap for the "About" form
	res 'WBMP', bmIcon2, "execyn.bmp"

; bitmaps for the arrows
	res 'WBMP', bmArrow1, "arrow1.bmp"
	res 'WBMP', bmArrow2, "arrow2.bmp"
	res 'WBMP', bmArrow3, "arrow3.bmp"
	res 'WBMP', bmArrow4, "arrow4.bmp"
	res 'WBMP', bmArrow5, "arrow5.bmp"
	res 'WBMP', bmArrow6, "arrow6.bmp"
	res 'WBMP', bmArrow7, "arrow7.bmp"
	res 'WBMP', bmArrow8, "arrow8.bmp"

; 'pref' resource. Defines app launch flags, stack and heap size

;        res 'pref', kidrPREF
;        dc.w    sysAppLaunchFlagNewStack|sysAppLaunchFlagNewGlobals|sysAppLaunchFlagUIApp|sysAppLaunchFlagSubCall
;        dc.l    $1000                           ; stack size
;        dc.l    $1000                           ; heap size

; Form resource

	res 'tFRM', mainForm, "tfrm03e8.bin"
	res 'tFRM', infoForm, "tfrm044c.bin"

; Version resource

	res 'tver', kidrTVER
      dc.b    '0.9', 0

; Menu resource

	res 'MBAR', mainMenu, "mbar041a.bin"



